#!/bin/bash

limit_iface() {
  local dev=$1
  local shape_rate=$2  # "guaranteed"
  local shape_ceil=$3  # "upper limit"

  tc qdisc delete dev "${dev}" root 2>/dev/null
  tc qdisc add dev "${dev}" root handle 1: htb default 0x11
  # basically no limit for default traffic
  tc class add dev "${dev}" parent 1: classid 1:11 htb rate 10Gbit ceil 100Gbit quantum 100000
  # limit "bad" (not whitelisted) traffic
  tc class add dev "${dev}" parent 1: classid 1:12 htb prio 1 rate "${shape_rate}" ceil "${shape_ceil}"
  # use "codel" qdisc for both classes, but with larger queue for default traffic
  tc qdisc add dev "${dev}" parent 1:11 handle 11: codel limit 20000
  tc qdisc add dev "${dev}" parent 1:12 handle 12: codel

  # sort into bad class based on netfilter mark (if bit 0x10 is set)
  tc filter add dev "${dev}" parent 1: prio 1 basic match 'meta(nf_mark mask 0x10 eq 0x10)' classid 1:12
}

uplink=$1
downlink=$2

if [ -z "${uplink}" -o -z "${downlink}" ]; then
  echo >&2 "Missing uplink and downlink interface names"
  exit 1
fi

limit_iface "${uplink}" "1Mbit" "1Mbit"
limit_iface "${downlink}" "1Mbit" "1Mbit"
