from __future__ import annotations

import argparse
import dataclasses

import hypercorn.config
import hypercorn.trio.run
import hypercorn.utils

import capport.config


def run(config: hypercorn.config.Config) -> None:
    sockets = config.create_sockets()
    assert config.worker_class == "trio"

    hypercorn.trio.run.trio_worker(config=config, sockets=sockets)

    for sock in sockets.secure_sockets:
        sock.close()
    for sock in sockets.insecure_sockets:
        sock.close()


@dataclasses.dataclass
class CliArguments:
    config: str | None

    def __init__(self):
        parser = argparse.ArgumentParser()
        parser.add_argument("--config", "-c")
        args = parser.parse_args()

        self.config = args.config


def main() -> None:
    args = CliArguments()
    _config = capport.config.Config.load_default_once(filename=args.config)

    hypercorn_config = hypercorn.config.Config()
    hypercorn_config.application_path = "capport.api.app"
    hypercorn_config.worker_class = "trio"
    hypercorn_config.bind = [f"127.0.0.1:{_config.api_port}"]

    if _config.server_names:
        hypercorn_config.server_names = _config.server_names
    elif not _config.debug:
        raise Exception(
            "production setup requires server-names in config (list of accepted hostnames in http requests)"
        )

    run(hypercorn_config)
