from __future__ import annotations

import typing

from .protobuf import message_pb2


def _message_to_variant(self: message_pb2.Message) -> typing.Any:
    variant_name = self.WhichOneof("oneof")
    if variant_name:
        return getattr(self, variant_name)
    return None


def _make_to_message(oneof_field):
    def to_message(self) -> message_pb2.Message:
        msg = message_pb2.Message(**{oneof_field: self})
        return msg

    return to_message


def _monkey_patch():
    g = globals()
    g["Message"] = message_pb2.Message
    message_pb2.Message.to_variant = _message_to_variant
    for field in message_pb2._MESSAGE.oneofs_by_name["oneof"].fields:
        type_name = field.message_type.name
        field_type = getattr(message_pb2, type_name)
        field_type.to_message = _make_to_message(field.name)
        g[type_name] = field_type


# also re-exports all message types
_monkey_patch()
# not a variant of Message, still re-export
MacState = message_pb2.MacState
